# -*- coding: utf-8 -*-
#    Paroli
#
#    copyright 2008 OpenMoko
#
#    This file is part of Paroli.
#
#    Paroli is free software: you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Paroli is distributed in the hope that it will be useful, but
#    WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Paroli.  If not, see <http://www.gnu.org/licenses/>.


from logging import getLogger
logger = getLogger('applications.inout')

from os.path import join, dirname
from tichy.service import Service
from tichy.application import Application
from tichy.tasklet import Wait, WaitFirst
from paroli.gui import ElementaryListWindow, ElementaryList

class InOutCallLog(Application):
    name = 'CallLog'
    icon = 'icon.png'
    category = 'launcher' # So that we see the app in the launcher
    launcher_info = ['GSM',"missed_call_count"]

    def run(self, parent=None, standalone=False):
        ##to be standardized
        self.edje_file = join(dirname(__file__), 'i-o.edj')

        self.gsm_service = Service.get('GSM')
        self.callLogs = self.gsm_service.logs

        ##get message service and list of all messages
        self.contact_service = Service.get('Contacts')
        self.contacts = self.contact_service.contacts
        ##sort contact by date
        def comp2(m1, m2):
            return cmp(str(m1.name).lower(), str(m2.name).lower())

        self.contacts.sort(comp2)

        ##generate app-window
        self.window = ElementaryListWindow(self.edje_file, "main", "list", None, None, True)
        self.edje_obj = self.window.main_layout

        def comp(m1, m2):
            return cmp(m2.timestamp, m1.timestamp)

        #  We use the Call object to as items in the call log
        self.list_label = [('label', 'number'), ('subtext', 'description')]
        self.item_list = ElementaryList(self.callLogs, self.window, self.edje_file, "item", self.list_label, comp)

        self.edje_obj.add_callback("to_edit_mode", "*", self.to_edit_mode)
        self.edje_obj.add_callback("to_default_mode", "*", self.to_default_mode)
        
        ## close the Tele app, with the back button (signal, source, method)
        self.edje_obj.add_callback("back", "edje", self.signal) 

        self.oid = self.contacts.connect('inserted', self.item_list._redraw_view)

        if len(self.callLogs) == 0:
            self.edje_obj.edje.signal_emit("ListEmpty", "python")
            self.oid2 = self.callLogs.connect('inserted', self.restore_edit)

        self.item_list.add_callback("new_call", "*", self.create_call)
        self.item_list.add_callback("new_msg", "*", self.create_msg)
        self.item_list.add_callback("save_number", "*", self.create_contact)

        self.item_list.elm_window = self.window.window

        parent.emit("unblock")

        i, args = yield WaitFirst(Wait(self.window, 'back'),
	                              Wait(self.window, 'delete_request'),
                                  Wait(self.window.window,'closing'))

        for i in self.callLogs:
            i.missed = False
            i.check()

        self.callLogs.emit("modified")

        if i != 2:
            self.contacts.disconnect(self.oid)
            self.window.delete()
            del self.item_list

    def signal(self, emission, signal, source):
        """ Callback function. It invokes, when the "back" button clicked."""
        logger.info("i-o2.py:signal() emmision: %s, signal: %s, source: %s", 
                    str(emission), str(signal), str(source))
        self.window.emit('back')
    
    def restore_edit(self, *args, **kargs):
        self.edje_obj.edje.signal_emit("ListFilled", "python")

    def to_edit_mode(self, emission, source, param):
        for item in self.item_list.items:
            item[1].signal_emit("to_edit_mode", "*")

    def to_default_mode(self, emission, source, param):

        deleting = []

        for i in range(len(self.item_list.items)):
            if self.item_list.items[i][1].part_text_get("delete-flag") == "1":
                deleting.append(self.item_list.items[i][0])
            else:
                logger.debug('to_default_mode %s', self.item_list.items[i][1].part_text_get("delete-flag"))

        deleting.reverse()

        self.item_list.model.remove_group(deleting)

        for item in self.item_list.items:
                item[1].signal_emit("to_default_mode", "*")

        if len(self.callLogs) == 0:
            self.edje_obj.edje.signal_emit("ListEmpty", "python")

    def create_call(self, emission, source, param, callLog):
        number = callLog[0].number.value
        name = unicode(callLog[0])
        caller_service = Service.get('TeleCaller2')
        caller_service.call("window", number, name).start()

    def create_msg(self, emission, source, param, callLog):
        service = Service.get('MessageCreate')
        service.write(self.window, 'reply', callLog[0].number).start()

    def create_contact(self, emission, source, param, callLog):
        service = Service.get('ContactCreate')
        service.create(self.window, callLog[0].number).start()

    def printer(self, *args, **kargs):
        logger.debug('printer %s %s', args, kargs)
